# Script.R: R-Code to reproduce the results of

# Eduard Szöcs, Ben J. Kefford, Ralf B. Schäfer (2012). 
# Is there an interaction of the effects of salinity and pesticides on 
# the community structure of macroinvertebrates? 
# Science of the Total Environment

# Eduard Szöcs
# University Koblenz-Landau
# Institute for Environmental Sciences
# Fortstr. 7
# 76829 Landau
# GERMANY
# szoe8822@uni-landau.de

## The code has been written so that it can be applied to data 
# with the same structure as the supplementary file  'Supplement.zip':
# See README.txt for details

############################
### Setup project Structure
############################

### path to project
## This has to be changed
pr <- "/home/edisz/Documents/Uni/Projects/Szöcs_2012_STOTEN/Revision/Supplement/"

### paths to subfolders
datadir <- paste(pr, "data", sep = "")
srcdir <- paste(pr, "src", sep = "")


############################
### Load package and data
############################

### Load packages 
library(vegan)  														
library(reshape)

### load data
# waterchemistry
NAchem <- read.csv(file.path(datadir, "waterchem.csv"), 
                   sep = ";", header = TRUE)		
# waterbody
body <- read.csv(file.path(datadir, "habitat.csv"), 
                 sep = ";" , header = TRUE)
# tox-data
toxdat <- read.csv(file.path(datadir, "tu_max.csv"), 
                   sep = ";", header = TRUE)
# abundance data
spec.raw <- read.csv(file.path(datadir, "abundance.csv"), 
                     sep = ";", header = TRUE)  

# Seed for Random Number Generation 
set.seed(2012)


############################
### Defined Functions
############################

# Replace NAs
# a = vector, a.impute = impute with
imput.NA <- function (a, a.impute){
  ifelse (is.na(a), a.impute, a)			
}											

# Replace 0 by a * min value (except zero)
# x = vector to impute a = multiplyer for min value
imput.min <- function(x , a = 0.5){
  snd <- sort(unique(x))
  ifelse(x == 0, a * snd[2], x)
}										

# Variable clustering
# method = correlation coef, t = trigger
# plots a dendrogramm 
varclust <- function (data, method = "spearman", t = 0.7){		
  bcors <- cor(data, method = method)
  bcorsa <- abs(bcors)
  bcorsad <- as.dist(bcorsa)
  plot(hclust(1-bcorsad, method = "complete"), 
       ylab = "1 - Spearmans-coefficient")
  abline(h = 1-t, lwd = 2, col = "red", lty = 2)
}

############################
### Data Cleaning
############################
# delete Alkalinity due to high amount of missing data
site_info <- c("Site_Month", "Site", "Month")

NAchem <- NAchem[ ,-13]

# convert percentages
body$rif_perc = body$rif_perc*100  		
body$pool_perc = body$pool_perc*100

# replace flow class with weights
weights <- data.frame(class = c(0, 1, 2, 3, 4), 
                      weights = c(0, 0.1, 0.25, 0.5, 0.75))

# replace class with weights
body$Noflow <- weights$weights[match(body$Noflow, weights$class)]
body$Slow <- weights$weights[match(body$Slow, weights$class)]
body$Medium <- weights$weights[match(body$Medium, weights$class)]
body$Fast <- weights$weights[match(body$Fast, weights$class)]

# median velocities [m/s]
velo <- c(0.025, 10, 35, 60)
body$velocity <- rowSums(sweep(body[ ,c('Noflow', 'Slow', 'Medium', 'Fast')], 
                               2, velo, "*"))
# velocity as m/s
body$velocity <- 0.01 * body$velocity
### approx. discharge as depth*mean(width)*velo
body$discharge <- rowMeans(body[ ,c("max_width", "min_width")]) * 
  body$velocity * body$Depth

# remove weights
body$Noflow <- NULL
body$Slow <- NULL
body$Medium <- NULL
body$Fast <- NULL


# prepare tox-data
# maximum TU_Daphnia by sampling
tox <- cast(toxdat, Site_Month + Site + Month ~ Matrix, fun.aggregate = max, value = "TU_Daphnia", fill = NA)


# merge water quality with habitat data
env <- merge(NAchem, body, by = site_info)
# merge tox
envdata <- merge(env,tox, by = site_info, all.x = TRUE)	

# Substitution of zeros by 1/2 LOD# see Clarke 1998
envdata$Nh4 <- imput.min(envdata$Nh4, 0.5)			
envdata$NO2 <- imput.min(envdata$NO2, 0.5)			
envdata$NO3 <- imput.min(envdata$NO3, 0.5)		
envdata$PO4 <- imput.min(envdata$PO4, 0.5)		

# Replace no detections TU with min val
envdata$Sediment <- imput.NA(envdata$Sediment, 
                                        min(envdata$Sediment, na.rm = TRUE))
envdata$TRIMPS <- imput.NA(envdata$TRIMPS, 
                                      min(envdata$TRIMPS, na.rm = TRUE))
envdata$TRIMPS_eq <- imput.NA(envdata$TRIMPS_eq, 
                                         min(envdata$TRIMPS_eq, na.rm = TRUE)) 
envdata$Water <- imput.NA(envdata$Water, 
                                     min(envdata$Water, na.rm = TRUE))


# Computation of maxTU
# toxicity variables
toxvar <- c('Sediment', 
            'TRIMPS',
            'TRIMPS_eq',
            'Water')
# compute max
maxTU <- apply(envdata[ ,toxvar],1,max)
# append
envdata <- data.frame(envdata[ ,!names(envdata) %in% toxvar], maxTU)

# transformations
hist(envdata$T)						# T good approximation of normal distribution
hist(envdata$pH)					# pH good approximation of normal distribution
hist(envdata$Cond)					# left skew, wide range=> log transformation
hist(log10(envdata$Cond))				
log_Cond <- log10(envdata$Cond)
hist(envdata$oxygen)				# good approximation of normal distribution
hist(envdata$Nh4)					# left skew => log transformation
hist(log10(envdata$Nh4))			# log Nh4 good approximation of normal distribution
log_Nh4 <- log10(envdata$Nh4)
hist(envdata$NO2)					
hist(log10(envdata$NO2))			# only a few detections...
log_NO2 <- log10(envdata$NO2)
hist(envdata$NO3)					
hist(log10(envdata$NO3))			# only a few detections...
log_NO3 <- log10(envdata$NO3)
hist(envdata$PO4)					    # left skew = log transformation
hist(log10(envdata$PO4))			# good approximation of normal distribution
log_PO4 <- log10(envdata$PO4)
hist(envdata$Turb)					# moderat left skew
hist(log10(envdata$Turb))				# log transformation  good approximation of normal distribution
log_Turb <- log10(envdata$Turb)
hist(envdata$Depth)					
log_maxTU <- log10(envdata$maxTU)	# TU as log
hist(envdata$velocity)
hist(log10(envdata$velocity))  # Velocity as log
log_velocity <- log10(envdata$velocity)
hist(envdata$discharge)
hist(log10(envdata$discharge))   # Discharge as log
log_discharge <- log10(envdata$discharge)

# Table 1
apply(envdata[ ,-c(1:3)], 2, range, na.rm = TRUE)

# drop vars
drop <- c('Cond', 'Nh4', 'NO2', 'NO3', 'PO4', 'Turb', 'velocity', 'discharge', 'maxTU')
envdata2 <- data.frame(envdata[ ,!names(envdata) %in% drop], 
                       log_Cond, log_Nh4, log_NO2, log_NO3, 
                       log_PO4, log_Turb, log_maxTU, log_velocity, log_discharge) 

envdata2$Site <- factor(envdata2$Site)

# Deterministic Regression Imputation		
# NA imputed by predicts of linear regression
apply(envdata2, 2, function(x) sum(is.na(x))) 

# Temperature
mod_t_f <- lm(T ~ Site + Month + pH + Depth + max_width + min_width +
  rif_perc + pool_perc + Bedrock + Boulder + Cobble + Pebble + Gravel + Sand + Clay.silt +
  log_Cond + log_Nh4 + log_NO3 + log_Turb + log_maxTU + log_velocity + log_discharge, 
              data = envdata2)
mod_t_n <- lm(T ~ 1, data = envdata2)
mod_t <- step(mod_t_f, scope = formula(mod_t_n))   # backward selection 	
pred_T <- predict(mod_t, envdata2)			# predicts 
t_imp <- round(imput.NA(envdata2$T, pred_T), digits=1)   # Replace NAs by predicts
envdata2$T = t_imp							       

# oxygen
mod_o_f <- lm(oxygen ~ Site + Month + pH + Depth + max_width + min_width +
  rif_perc + pool_perc + Bedrock + Boulder + Cobble + Pebble + Gravel + Sand + Clay.silt +
  log_Cond + log_Nh4 + log_NO3 + log_Turb + log_maxTU + log_velocity + log_discharge , data = envdata2)
mod_o_n <- lm(T ~ 1, data = envdata2)
mod_o <- step(mod_o_f, scope = formula(mod_o_n))
summary(mod_o)
pred_o <- predict(mod_o, envdata2)
o_imp <- round(imput.NA(envdata2$oxygen, pred_o), digits=1)
envdata2$oxygen = o_imp	

# NO2			
mod_no2_f = lm(log_NO2 ~ Site + Month + pH + Depth + max_width + min_width +
  rif_perc + pool_perc + Bedrock + Boulder + Cobble + Pebble + Gravel + Sand + Clay.silt +
  log_Cond + log_Nh4 + log_NO3 + log_Turb + log_maxTU + log_velocity + log_discharge , data = envdata2)
mod_no2_n = lm(log_NO2 ~ 1, data = envdata2) 
mod_no2 = step(mod_no2_f, scope = formula(mod_no2_n))	
summary(mod_no2)
pred_no2 <- predict(mod_no2, envdata2)	
no2_imp <- round(imput.NA(envdata2$log_NO2, pred_no2), digits = 2)	
envdata2$log_NO2 = no2_imp					

# PO4
mod_po4_f = lm(log_PO4 ~ Site + Month + pH + Depth + max_width + min_width +
  rif_perc + pool_perc + Bedrock + Boulder + Cobble + Pebble + Gravel + Sand + Clay.silt +
  log_Cond + log_Nh4 + log_NO3 + log_Turb + log_maxTU + log_velocity + log_discharge , data = envdata2)
mod_po4_n = lm(log_PO4 ~ 1, data = envdata2) 
mod_po4 = step(mod_po4_f, scope = formula(mod_no2_n))	
summary(mod_po4)
pred_po4 <- predict(mod_po4, envdata2)	
po4_imp <- round(imput.NA(envdata2$log_PO4, pred_no2), digits = 2)	
envdata2$log_PO4 = po4_imp
sum(apply(envdata2, 2, function(x) sum(is.na(x))))   # No more NAs



# variable clustering
varclust(envdata2[ ,!names(envdata2) %in% c(site_info, 'Riffle')])		
# remove rif_perc
varclust(envdata2[ ,!names(envdata2) %in% c(site_info, 'Riffle', "rif_perc")])
# remove max_width
varclust(envdata2[ ,!names(envdata2) %in% c(site_info, 'Riffle', "rif_perc",
                                            "max_width")])
varclust(envdata2[ ,!names(envdata2) %in% c(site_info, 'Riffle', "rif_perc",
                                            "max_width", "Gravel")])
varclust(envdata2[ ,!names(envdata2) %in% c(site_info, 'Riffle', "rif_perc",
                                            "max_width", "Gravel", "Cobble")])
varclust(envdata2[ ,!names(envdata2) %in% c(site_info, 'Riffle', "rif_perc",
                                            "max_width", "Gravel", "Cobble",
                                            "Pebble")])
varclust(envdata2[ ,!names(envdata2) %in% c(site_info, 'Riffle', "rif_perc",
                                            "max_width", "Gravel", "Cobble",
                                            "Pebble", "min_width", "Clay.silt")])
varclust(envdata2[ ,!names(envdata2) %in% c(site_info, 'Riffle', "rif_perc",
                                            "max_width", "Gravel", "Cobble",
                                            "Pebble", "min_width", "Clay.silt",
                                            "Depth")])

# resulting environmental variables
fin_envdata <- envdata2[ ,!names(envdata2) %in% c('Riffle', "rif_perc",
                                                  "max_width", "Gravel", "Cobble",
                                                  "Pebble", "min_width", "Clay.silt",
                                                  "Depth")]   # remove highly correlated variables


colnames(spec.raw) <- gsub("Family.", "", colnames(spec.raw))    # remove 'Family' in species names
specr <- spec.raw
head(spec.raw)

mat <- dim(specr[ ,!names(specr) %in% site_info])
sum(specr[ ,-c(1:3)] == 0) / (mat[1] * mat[2])   # percentage of zero entries


############################
### Interaction Pesticides & Salinity
############################

### Check for differences between streams with and without riffle samples
summary(aov(log_Cond ~ Riffle * Month, data = envdata2))
summary(aov(log_maxTU ~ Riffle * Month, data = envdata2))


### Raw data plots of 20 most abundant taxa #############
library(ggplot2)
specrm <- melt(specr,id = c("Site_Date", "Site", "Month"))
plotdf <- merge(specrm, fin_envdata, by.x = c("Site_Date", "Site", "Month"),
                by.y = c("Site_Month", "Site", "Month"))
take <- colnames(specr[ , -c(1:3)])[order(colSums(specr[ ,-c(1:3)]), decreasing=TRUE)][1:20]
subs2 <- plotdf[plotdf$variable %in% take, ]

ggplot(subs2, aes(x = log_Cond, y = value^0.25)) +
  geom_point() +
  facet_wrap(~variable, scales="free_y") +
  geom_smooth(se = FALSE, method = "loess")

ggplot(subs2, aes(x = log_maxTU, y = value^0.25)) +
  geom_point() +
  facet_wrap(~variable, scales = "free_y") +
  geom_smooth(se = FALSE, method = "loess")


#### September ################
##species
specr_sub <- specr[specr$Month == 9, ]        # take only data from September       	
spec_sub <- specr_sub[ ,-c(1:3)]^0.25         # 4th-root transformation of abundances
spec_sub <- data.frame(specr_sub[ ,c(1:3)], spec_sub)					
spec_sub <- spec_sub[order(spec_sub$Site), ]  # reorder
rownames(spec_sub) <- spec_sub$Site_Month

## environment
envr_sub <- fin_envdata[fin_envdata$Month == 9, ]   # take only data from September
s_envr_sub <- scale(envr_sub[ ,-c(1:3)])				    # scale environmental variables
s_env_sub <- data.frame(envr_sub[ ,c(1:3)], s_envr_sub)
s_env_sub <- s_env_sub[order(s_env_sub$Site), ]

## Db-RDA with interaction
mod_sub_int <- capscale(spec_sub[ ,-c(1:3)] ~ log_maxTU * log_Cond, data = s_env_sub, distance = "bray")
## Permutation test of marginal effects of constraints
anova(mod_sub_int, by="margin", step = 1000, perm.max = 1000)   # ns
mod_sub <- update(mod_sub_int, .~. -log_maxTU:log_Cond)         # remove interaction from model
## Permutation test of marginal effects of constraintes
anova(mod_sub, by="margin", step = 1000, perm.max = 1000)        


### November  ####################
##species
specr_sub <- specr[specr$Month == 11, ]             
spec_sub <- specr_sub[ ,-c(1:3)]^0.25
spec_sub <- data.frame(specr_sub[ ,c(1:3)], spec_sub)					
spec_sub <- spec_sub[order(spec_sub$Site), ]
rownames(spec_sub) <- spec_sub$Site_Month

## environment
envr_sub <- fin_envdata[fin_envdata$Month == 11, ]
s_envr_sub <- scale(envr_sub[ ,-c(1:3)])				
s_env_sub <- data.frame(envr_sub[ ,c(1:3)], s_envr_sub)
s_env_sub <- s_env_sub[order(s_env_sub$Site), ]

mod_sub_int <- capscale(spec_sub[ ,-c(1:3)] ~ log_maxTU * log_Cond, data = s_env_sub, distance = "bray")
anova(mod_sub_int, by="margin", step = 1000, perm.max = 1000)
mod_sub <- update(mod_sub_int, .~. -log_maxTU:log_Cond)
anova(mod_sub, by="margin", step = 1000, perm.max = 1000)


### Feb #########################
##species
specr_sub <- specr[specr$Month == 2, ]             
spec_sub <- specr_sub[ ,-c(1:3)]^0.25
spec_sub <- data.frame(specr_sub[ ,c(1:3)], spec_sub)					
spec_sub <- spec_sub[order(spec_sub$Site), ]
rownames(spec_sub) <- spec_sub$Site_Month

## environment
envr_sub <- fin_envdata[fin_envdata$Month == 2, ]
s_envr_sub <- scale(envr_sub[ ,-c(1:3)])				
s_env_sub <- data.frame(envr_sub[ ,c(1:3)], s_envr_sub)
s_env_sub <- s_env_sub[order(s_env_sub$Site), ]

mod_sub_int <- capscale(spec_sub[ ,-c(1:3)] ~ log_maxTU * log_Cond, data = s_env_sub, distance = "bray")
anova(mod_sub_int, by="margin", step = 1000, perm.max = 1000)
mod_sub <- update(mod_sub_int, .~. -log_maxTU:log_Cond)
anova(mod_sub, by="margin", step = 1000, perm.max = 1000)



#### Plot Figure 2
##species
specr_yr <- specr                            # copy species  		
spec_yr <- specr_yr[ ,-c(1:3)]^0.25          # fourth-root transformation
spec_yr <- data.frame(specr_yr[ ,c(1:3)], spec_yr)					
spec_yr <- spec_yr[order(spec_yr$Site_Date),]
rownames(spec_yr) <- spec_yr$Site_Date

## environment
env_yr.1 <- fin_envdata
s_env_yr.1 = scale(env_yr.1[ ,-c(1:3)])				 # scale environmental data
s_env_yr = data.frame(env_yr.1[ ,c(1:3)], s_env_yr.1)
s_env_yr$Month[s_env_yr$Month == 2] <- 14      #  replace February by Month 14
s_env_yr <- s_env_yr[order(s_env_yr$Site_Month),]
# Db-RDA with Toxicity and Salinity constraints and removing a Time-Effect
mod.yr3 <- capscale(spec_yr[ ,-c(1:3)] ~ log_maxTU + log_Cond + Condition(Month), data = s_env_yr, distance = "bray")
plot(mod.yr3, display = "species")


pubmod <- mod.yr3    # copy model
#species subset 
subs <- apply(spec_yr[ ,-c(1:3)]^4, 2, max) >= 10   # only species with a maximum abundance higher then 10
## Specify groups
phylum <- c("Crustacea", "Plecoptera", "Ephemeroptera", "Ephemeroptera", "Trichoptera", "Trichoptera", "Crustacea", "Diptera", "Odonata", "Trichoptera", "Mollusca", "Heteroptera", "Diptera", "Coleoptera", "Trichoptera", "Coleoptera", "Plecoptera", "Hirudinea", "Plecoptera", "Coleoptera","Trichoptera", "Trichoptera", "Trichoptera", "Coleoptera", "Crustacea", "Crustacea", "Trichoptera", "Ephemeroptera", "Mollusca", "Heteroptera", "Ephemeroptera", "Mollusca", "Mollusca", "Odonata", "Coleoptera", "Crustacea", "Diptera", "Mollusca", "Odonata", "Odonata", "Odonata", "Diptera", "Heteroptera", "Mites", "Oligochaeta")
table(phylum)
phylum[phylum %in% c("Oligochaeta", "Hirudinea", "Mites", "Odonata", "Heteroptera")] <- "others"

pubmod[["colsum"]] <- pubmod[["colsum"]][subs]   # subset species
pubmod[["CCA"]][["v"]] <- pubmod[["CCA"]][["v"]][subs, ]
pubmod[["CCA"]][["v.eig"]] <- pubmod[["CCA"]][["v.eig"]][subs, ]
pubmod[["CA"]][["v"]] <- pubmod[["CA"]][["v"]][subs, ]
pubmod[["CA"]][["v.eig"]] <- pubmod[["CA"]][["v.eig"]][subs, ]
rownames(pubmod[["CCA"]][["biplot"]]) <- c("TU max", "Conductivity")   # Replace names of Constraints

plot(pubmod, display = c("sp", "cn"), xlim = c(-1.3, 1), ylim = c(-0.5, 0.5))   #plot ordination
## add hulls for Groups to plot
ordihull(pubmod, group = phylum,  display = "sp", show.groups = c("Crustacea"), lwd = 2, lty = "dashed")
ordihull(pubmod, group = phylum,  display = "sp", show.groups = c("Ephemeroptera"), lwd = 2, lty = "dotted")
ordihull(pubmod, group = phylum,  display = "sp", show.groups = c("Trichoptera"), lwd = 2 , lty = "solid")

## add axis labelling and legend
summm <- summary(pubmod)
xtext <- paste("(", round(summm$concont[[1]][2, 1] * 100, 2), " % of fitted, ", 
               round(summm$cont[[1]][2, 1] * 100, 2), " % of total variation)", sep = "")
ytext <- paste("(", round(summm$concont[[1]][2, 2] * 100, 2), " % of fitted, ", 
               round(summm$cont[[1]][2, 2] * 100, 2), " % of total variation)", sep = "")
mtext(xtext, 1, line = 4)
mtext(ytext, 2, line = 2)
leg.txt <- c("Crustacea", "Ephemeroptera", "Trichoptera")
legend("topleft", leg.txt, lty = c("dashed", "dotted", "solid"), cex = 0.8, x.intersp = 0.5, y.intersp = 0.8, bty = "n")


########################################
#### Forward selection of environmental variables
#######################################

##species
specr_yr <- specr          # copy species   	
spec_yr <- specr_yr[ ,-c(1:3)]^0.25     # fourth-root transformation
spec_yr <- data.frame(specr_yr[ ,c(1:3)], spec_yr)					
spec_yr <- spec_yr[order(spec_yr$Site_Date),]
rownames(spec_yr) <- spec_yr$Site_Month

## environment
env_yr.1 <- fin_envdata
s_env_yr.1 = scale(env_yr.1[ ,-c(1:3)])				# scale environmental data
s_env_yr = data.frame(env_yr.1[ ,c(1:3)], s_env_yr.1)
s_env_yr$Month[s_env_yr$Month == 2] <- 14      #  replace February by Month 14
s_env_yr <- s_env_yr[order(s_env_yr$Site_Month), ]
s_env_yr <- s_env_yr[ , -c(1,2)]
# Full db-RDA model with all environmental variables
modf <- capscale(formula = spec_yr[, -c(1:3)] ~ . - Month + Condition(Month), 
                 data = s_env_yr,
                 distance = "bray")
# Null Model
modn <- capscale(spec_yr[ ,-c(1:3)] ~ 1 + Condition(Month) , data = s_env_yr, distance = "bray")
# Perform Forward-Selection ala Blanchet (2008)
modr <- ordiR2step(modn, modf, pstep = 1000, by = "margin")
# Permutation test of variables
aov <- anova(modr, by = "margin", step = 1000, perm.max=1000)
aov
# Cumulative Variance explained
cumsum(aov$Var) / sum(aov$Var)

### Plot Figure 1
subs <- apply(spec_yr[ ,-c(1:3)]^4, 2, max) >= 10
modr[["colsum"]] <- modr[["colsum"]][subs]   # subset species
modr[["CCA"]][["v"]] <- modr[["CCA"]][["v"]][subs, ]
modr[["CCA"]][["v.eig"]] <- modr[["CCA"]][["v.eig"]][subs, ]
modr[["CA"]][["v"]] <- modr[["CA"]][["v"]][subs, ]
modr[["CA"]][["v.eig"]] <- modr[["CA"]][["v.eig"]][subs, ]
rownames(modr[["CCA"]][["biplot"]]) <- c("Conductivity", "Pool %", "Sand",
                                         "Discharge", "Velocity", "Boulder", "TU max")
plot(modr,display = c("sp", "cn"))




